%Forecaster
%Chris Rackauckas
%Estimates the specified ARIMA(p,d,q) model and generates a forecast.
%Can be used to analyze an ARIMA forcast against validationData data by setting a
%portion of your data as the validation data. If you wish to only forecast,
%simply turn the validation step off (VALIDATE=0). One can specify a second
%ARIMA model estimated using a subset of the dataset by setting TWOARIMA=1 
%and the program will plot the estimation errors of both models for
%comparison.

%Sets up the data
%For user specification
data = importdata('CPILevelq.xls'); % 1983:4 = 148.
fullData = data.data.Sheet1(:,3); % For this data set, 49 = 1959:1. 240 = 2006:4. End is 264
fullData = log(fullData)    %For this dataset we wanted to log-transform the data
data = fullData(49:240);    %Data used for the ARIMA estimation
TWOARIMA = 1;               %Toggles estimation for a subset of the data
VALIDATE = 1;               %Toggles validation step
p = 3;
d = 1;
q = 4;

if VALIDATE
   validationData = fullData(241:264);   %Data for validation step
end

if TWOARIMA
   %For the sample data, estimates the second model from the post 1983:4 data
   dataSubset =  data(148:end);
end

%Rest of the program

%Estimates the model from the data
forecastLength = 28;
model = arima(p,d,q);


esti = estimate(model,data);


[fore MSE] = forecast(esti,forecastLength,'Y0',data)



%Estimates the ARIMA using a subset of the data.
if TWOARIMA
    esti = estimate(model,data(148:end));
    [fore2 MSE2] = forecast(esti,forecastLength,'Y0',data(148:end));
end



%Initialize
beginF = length(data)+1;
endF = beginF+forecastLength-1;

%Computes 95%
lower = fore - 1.96*sqrt(MSE);
upper = fore + 1.96*sqrt(MSE);

%Plots
figure(1)
plot(data,'Color',[.7,.7,.7]);
hold on
h1 = plot(beginF:endF,lower,'r:','LineWidth',2);
plot(beginF:endF,upper,'r:','LineWidth',2)
h2 = plot(beginF:endF,fore,'k','LineWidth',2);

if VALIDATE
    h3 = plot(beginF:endF-4,validationData,'g','LineWidth',2);
    legend([h1 h2,h3],'95% Interval','Forecast','Validation Data',...
	     'Location','NorthWest')
else
    legend([h1 h2],'95% Interval','Forecast',...
	     'Location','NorthWest')
end

title('Forecast')
hold off

sprintf('here')

%Plots zoomed in pict
beginF=21;
endF = beginF+forecastLength-1;
figure(2)
plot(data(end-20:end),'Color',[.7,.7,.7]);
hold on
h1 = plot(beginF:endF,lower,'r:','LineWidth',2);
plot(beginF:endF,upper,'r:','LineWidth',2)
h2 = plot(beginF:endF,fore,'k','LineWidth',2);

if VALIDATE
    h3 = plot(beginF:endF-4,validationData,'g','LineWidth',2);
    legend([h1 h2,h3],'95% Interval','Forecast','Validation Data',...
	     'Location','NorthWest')
else
    legend([h1 h2],'95% Interval','Forecast',...
	     'Location','NorthWest')
end

title('Forecast Zoomed In')
hold off

%Plots forcast errors
fore = fore(1:end-4);
errors = fore - validationData;

figure(3)
hold on 
h1 = plot(errors,'k','LineWidth',2);

if TWOARIMA
    fore2 = fore2(1:end-4);
    errors2 = fore2 - validationData;
    h2 = plot(1:length(fore),errors2,'g','LineWidth',2);
    legend([h1 h2],'Full Data Forecast','Post-1984 Forecast',...
	     'Location','NorthWest');
else
    legend([h1],'Forecast',...
	     'Location','NorthWest');
end

title('Errors')
hold off

